#ifndef _USB_H    /* Guard against multiple inclusion */

#define _USB_H

#define MAX_USB_BUFFER_SIZE 512
#define USB_EP0_WAIT_TIMEOUT 1000

typedef struct
{
    int baud;
    short stop_bits;
    short parity_bits;
    short data_bits;
} USB_LINE_CODING;

typedef struct
{
    volatile unsigned short rx_num_bytes;
    volatile unsigned short tx_num_bytes;
    volatile unsigned char tx_buffer[MAX_USB_BUFFER_SIZE];
    volatile unsigned char rx_buffer[MAX_USB_BUFFER_SIZE];
} USB_ENDPOINT;

typedef struct
{
    volatile unsigned char bmRequestType;
    volatile unsigned char bmRequest;
    volatile unsigned short wValue;
    volatile unsigned short wIndex;
    volatile unsigned short wLength;
} USB_TRANSACTION;

unsigned char USB_device_description[] = {
/* Descriptor Length						*/ 18, /* Always 18 or 0x12 */
/* DescriptorType: DEVICE					*/ 0x01,
/* bcdUSB (ver 2.0)							*/ 0x00,0x02,
/* bDeviceClass								*/ 0x00,
/* bDeviceSubClass							*/ 0x00,
/* bDeviceProtocol							*/ 0x00,
/* bMaxPacketSize0							*/ 0x40, /* Always 64 or 0x40 for High Speed USB */
/* idVendor									*/ 0xD8,0x04, /* e.g. - 0x04D8 - Microchip VID */
/* idProduct								*/ 0x0A,0x00, /* e.g. - 0x000A */
/* bcdDevice								*/ 0x00,0x02, /* e.g. - 02.00 */
/* iManufacturer							*/ 0x01,
/* iProduct									*/ 0x02,
/* iSerialNumber							*/ 0x03, 
/* bNumConfigurations						*/ 0x01
};

unsigned char USB_address = 0;
volatile char USB_set_address = 0;
volatile char USB_line_control_state = 0;

USB_ENDPOINT EP[3];
USB_TRANSACTION USB_transaction;
USB_LINE_CODING USB_line_coding;

void USB_init(void *receive_callback);
void USB_connect();
void USB_send_EP2(volatile unsigned char *buffer, int size);
void (*USB_RECEIVE_CALLBACK)(volatile unsigned char *buffer, int length);

#endif /* _USB_H */